/*
エンコーダ値を管理するモジュール
・一定間隔でエンコーダ値を更新する
・エンコーダの読み取り値と積算値を提供する。
・エンコーダ積算値をリセットする。
2014/07/13	by S.Ushimaru

エンコーダの読み取りを3つの変更
全ての累積分の値も記録するように追加
2014/12/12 五十嵐
*/

#include <stdio.h>
#include <sys/io.h>
#include <unistd.h>
#include <pthread.h>
#include "encoder.h"
#include "device.h"
#include "./../hardware.h"


//static struct encoder_data encoder_data;
static pthread_t thread_id;
static int encoder_left=0;
static int encoder_right=0;
static int encoder_back=0;
static int encoder_left_sum=0;
static int encoder_right_sum=0;
static int encoder_back_sum=0;

static int encoder_left_sum_cum=0;
static int encoder_right_sum_cum=0;
static int encoder_back_sum_cum=0;//累積合計

void encoder_reset(){

	//printf("encoder reset called!\n");
        encoder_left_sum = 0;
        encoder_right_sum = 0;
        encoder_back_sum = 0;
}
void encoder_all_reset(){
  	encoder_left_sum_cum = 0;
	encoder_right_sum_cum = 0;
	encoder_back_sum_cum = 0;
}
void *encoder_read(void *t){

	int ret_val;
	char enc_data[3];//エンコーダ3つに変更
	char dadd;
        char *dumy;
        dumy = &dadd;

	int fd_encoder;
	fd_encoder=device_encoder_get();//デバイス番号の受け取り

	while(1){

        	ret_val=read(fd_encoder, enc_data, 3);//読み取り

//moto
      /*  	encoder_left = enc_data[0];
        	encoder_right = enc_data[1];
        	encoder_back = enc_data[2];
      */
		encoder_left = enc_data[1];
        	encoder_right = enc_data[2];
        	encoder_back = enc_data[0];
		//printf("left=%5d    right=%5d\n",encoder_left, encoder_right);

        	ret_val=write(fd_encoder,dumy,1); //エンコーダのリセット
		
			//読み取り値の合計
        	//encoder_right *= -1;  //方向反転への対応
        	encoder_left_sum += encoder_left;
        	encoder_right_sum += encoder_right;
        	encoder_back_sum += encoder_back;
			//累積量
        	encoder_left_sum_cum += encoder_left;
        	encoder_right_sum_cum += encoder_right;
        	encoder_back_sum_cum += encoder_back;
		usleep((T_ENCODER -1) *1000 ); // 1ms はスケジューラの精度で遅れる


	}

	//printf("left=%5d    right=%5d\n",encoder_data->left, encoder_data->right);

	//return;

}

static int thread_create()
{
        pthread_attr_t attr;
        struct sched_param sched_param;
        int ret;
        int policy;

        pthread_attr_init(&attr);

        sched_param.sched_priority = 20;
        pthread_attr_setschedparam(&attr, &sched_param);
        pthread_setschedparam(pthread_self(), SCHED_FIFO , &sched_param);

        ret = pthread_create(&thread_id, &attr, encoder_read , (void *)2);

	 // 優先度20、ポリシーFIFOでスレッド生成
        pthread_getschedparam(pthread_self(), &policy, &sched_param);
        printf("policy = %s, priority =%d\n",
                (policy == SCHED_FIFO)  ? "SCHED_FIFO" :
                (policy == SCHED_RR)    ? "SCHED_RR" :
                (policy == SCHED_OTHER) ? "SCHED_OTHER" :
                "???",
                sched_param.sched_priority);

        return 0;
}

void encoder_get(struct encoder_data *encoder_data){
//エンコーダの情報を渡す
	encoder_data->left = encoder_left;
	encoder_data->right = encoder_right;
	encoder_data->back = encoder_back;
	encoder_data->left_sum = encoder_left_sum;
	encoder_data->right_sum = encoder_right_sum;
	encoder_data->back_sum = encoder_back_sum;
	return;

}

void encoder_all_get(struct encoder_all_data *encoder_all_data){
//累積値を渡す
	encoder_all_data->left_sum_cum = encoder_left_sum_cum;
	encoder_all_data->right_sum_cum = encoder_right_sum_cum;
	encoder_all_data->back_sum_cum = encoder_back_sum_cum;
	return;

}
void encoder_all_return(struct encoder_all_data *encoder_all_data){
//累積値を修正する
	encoder_left_sum_cum = encoder_all_data->left_sum_cum;
	encoder_right_sum_cum = encoder_all_data->right_sum_cum;
	encoder_back_sum_cum = encoder_all_data->back_sum_cum;
	return;

}

void encoder_create(){

        encoder_reset();
        thread_create();
        printf("thread_id=%d\n",(int)thread_id);

}

