/*
正体補正プログラム
					2014/12/12 五十嵐
超音波センサの番号は、次のようになっていることを前提としている。
	USS1:正面左側     USS2:正面右側		USS3:右側1	USS4:右側2

*/

#include <linux/errno.h>
#include <sys/io.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include <pthread.h>
#include <stdio.h>
#include "./../hardware.h"
#include "./../device/uss.h"
#include "./../run/run_ctrl.h"
#include "direction.h"


static float uss_dist_f = 11.0 ; //12.5; //超音波センサ間の距離[cm]
static float uss_dist_r = 11.0 ; //12.5; //超音波センサ間の距離[cm]

static float ref_angle;

/* 引数 dist：正体補正する物体までの距離 */
int direction_correct(int type){

	int uss_l; //left
	int uss_r; //right
	int diff_uss;
	float diff_max=10;
	int c_count=0;
	int c_count_max=10; // 正体補正を諦めるまでの回数
	int cur_dist;
	int ref_dist = 0; 
	//所定距離：距離補正に入る際の超音波センサの測定範囲
	int uss_max = 180; //所定距離の最大値[cm]//変更するかも
	int uss_min = 30; //所定距離の最小値[cm]//25くらいまで

	float rot_speed = 20.0; //正体補正時の回転速度[rad/s]
	float angle_coef = 0.8; //正体補正時の回転角度調整係数
	
	run_ctrl_set_speed(rot_speed);
	printf("Direction Correction Start !!1\n") ;



// 正体補正
	
	run_ctrl_reset();
	printf("正体補正スタート\n");

	while(1){
	
		//printf("-----------------------------------------------\n");
		//printf("correction count =%d\n",c_count);
		if(type == 0){//正面の超音波センサで正対補正
			uss_l=uss_get(3); // left
			usleep(50000);
			uss_r=uss_get(1); // right
			usleep(50000);
			printf("uss_l=%d   uss_r= %d\n",uss_l, uss_r) ;
		}else if(type == 1){//右側の超音波センサで正対補正
			uss_l=uss_get(4); // left
			usleep(50000);
			uss_r=uss_get(2); // right
			usleep(50000);
			printf("uss_r_l=%d   uss_r_r= %d\n",uss_l, uss_r) ;
		}else {return -1;}
		//一時的に回転減速域のゲインを変更する
		//run_ctrl_set_Kfr(1.0);

		diff_uss = uss_l - uss_r;
		cur_dist =(uss_l + uss_r)/2;
		//正体補正したと判断する条件 : USSの差を 0 とするか abs(1) とするか
		//if(diff_uss == 0) break;
		if( (diff_uss >= -1) && (diff_uss <=1) && uss_l <= 9000) break;

		if(diff_uss > diff_max) diff_uss = diff_max;
		else if (diff_uss < -diff_max) diff_uss = -diff_max;

		// 超音波センサ計測距離の差から回転角を計算、ただし angle_coef で調整
		//測定値の差/超音波同士の感覚
		if(type == 0){//正面の超音波センサで正対補正
			ref_angle = diff_uss / uss_dist_f * 180.0 /3.1415 * angle_coef ; // [deg]
		}else if(type == 1){
			ref_angle = diff_uss / uss_dist_r * 180.0 /3.1415 * angle_coef ; // [deg]
		}
		if(uss_l >= 9000 && uss_r >=9000){
						ref_angle = 15;
		}
		printf("diff_uss=%d      ref_angle=%6.1f\n",diff_uss, ref_angle);

		run_ctrl_create();

  		run_ctrl_set_shet_dist(-ref_angle);

		while(1){
			//printf("run roate loop \n");
			if(run_ctrl_get_endflag()==1){
				//run_ctrl_reset();
				break;
			}
			usleep(20000);
		}

		c_count++;

		if(c_count > c_count_max) break;

		usleep(100*1000); // 100ms待つ

	}
	run_ctrl_stop();
	run_ctrl_reset();
	if(c_count <= c_count_max){
//		return cur_dist;//距離を返す場合
		return 1;
	}else{
		return -1;
	}

	//return 0;

}

 int direction_bolok(int dist_f,int dist_r){
	int uss_l;
	int uss_r;
	int uss_f_l; //flont_left
	int uss_f_r; //flont_right
	int uss_r_l; //right_left
	int uss_r_r; //right_right
	int diff_uss;
	float diff_max=10;
	int cur_dist,cur_dist_f,cur_dist_r;
	int ref_dist = 0;//,ref_dist_r = 0; 
	//所定距離：距離補正に入る際の超音波センサの測定範囲
	int uss_max = 180; //所定距離の最大値[cm]//変更するかも
	int uss_min = 30; //所定距離の最小値[cm]//25くらいまで

	float str_speed = 10.0; //距離補正時の速度[cm/s]

	printf("Direction Correction Start !!1\n") ;

		// 距離の補正


	if(dist_f!=0){
		printf("y方向の距離補正スタート\n");

		ref_dist = dist_f;
	
		uss_l=uss_get(3); // flont_left
		usleep(50000);
		uss_r=uss_get(1); // flont_right
		usleep(50000);

		//両方のUSSの距離を信頼する場合
		if((uss_l > uss_min  && uss_l < uss_max ) && (uss_r > uss_min  && uss_r < uss_max)){
			cur_dist =(uss_l + uss_r)/2;
		} // 左のUSSの距離を信頼する場合
		else if(uss_l > uss_min  || uss_l < uss_max){
			cur_dist = uss_l;
		} // 右のUSSの距離を信頼する場合
		else if(uss_r > uss_min  || uss_r < uss_max){
			cur_dist = uss_r;
		}
		else{ //両方とも信頼出来ない場合
			cur_dist = 999;
		}

		printf("ref_dist=%d     cur_dist = %d\n",ref_dist,cur_dist);

		if(cur_dist!=999){
			run_ctrl_create();
			run_ctrl_set_speed(str_speed);
	
        		run_ctrl_set_xy_dist(0.0,(float)(cur_dist - ref_dist));
			while(1){
				//printf("run straight loop \n");
				if(run_ctrl_get_endflag()==1){
					//run_ctrl_reset();
					break;
				}
				usleep(20000);
			}
			run_ctrl_cancel();

		}else{
			printf("can't get distance using USS\n");
			return -1;
		}
		//usleep(1000*1000); //1秒待つ

	}


	if(dist_r!=0){
		printf("x方向の距離補正スタート\n");

		ref_dist = dist_r;
	
		uss_l=uss_get(4); // right_left
		usleep(50000);
		uss_r=uss_get(2); // right_right
		usleep(50000);

		//両方のUSSの距離を信頼する場合
		if((uss_l > uss_min  && uss_l < uss_max ) && (uss_r > uss_min && uss_r < uss_max)){
			cur_dist =(uss_l + uss_r)/2;
		} // 左のUSSの距離を信頼する場合
		else if(uss_l > uss_min  || uss_l < uss_max){
			cur_dist = uss_l;
		} // 右のUSSの距離を信頼する場合
		else if(uss_r > uss_min  || uss_r < uss_max){
			cur_dist = uss_r;
		}
		else{ //両方とも信頼出来ない場合
			cur_dist = 999;
		}

		printf("ref_dist=%d     cur_dist = %d\n",ref_dist,cur_dist);

		if(cur_dist!=999){
			run_ctrl_create();
			run_ctrl_set_speed(str_speed);
            		run_ctrl_set_xy_dist((float)(cur_dist - ref_dist),0.0);
			while(1){
				//printf("run straight loop \n");
				if(run_ctrl_get_endflag()==1){
					//run_ctrl_reset();
					break;
				}
				usleep(20000);
			}
			run_ctrl_cancel();

		}else{
			printf("can't get distance using USS\n");
			return -1;
		}
	//	usleep(1000*1000); //1秒待つ

	}

//補正ができたか確認
	uss_f_l=uss_get(3); // flont_left
	usleep(50000);
	uss_f_r=uss_get(1); // flont_right
	usleep(50000);
	uss_r_l=uss_get(4); // right_left
	usleep(50000);
	uss_r_r=uss_get(2); // right_right
	usleep(50000);

	//正面の両方のUSSの距離を信頼する場合
	if((uss_f_l > uss_min  || uss_f_l < uss_max ) && (uss_f_r > uss_min  || uss_f_r < uss_max)){
		cur_dist_f =(uss_f_l + uss_f_r)/2;
	} // 左のUSSの距離を信頼する場合
	else if(uss_f_l > uss_min  || uss_f_l < uss_max){
		cur_dist_f = uss_f_l;
	} // 右のUSSの距離を信頼する場合
	else if(uss_f_r > uss_min  || uss_f_r < uss_max){
		cur_dist_f = uss_f_r;
	}
	else{ //両方とも信頼出来ない場合
		cur_dist_f = 999;
	}

	//右側の両方のUSSの距離を信頼する場合
	if((uss_r_l > uss_min  || uss_r_l < uss_max ) && (uss_r_r > uss_min  || uss_r_r < uss_max)){
		cur_dist_r =(uss_r_l + uss_r_r)/2;
	} // 左のUSSの距離を信頼する場合
	else if(uss_r_l > uss_min  || uss_r_l < uss_max){
		cur_dist_r = uss_r_l;
	} // 右のUSSの距離を信頼する場合
	else if(uss_r_r > uss_min  || uss_r_r < uss_max){
		cur_dist_r = uss_r_r;
	}
	else{ //両方とも信頼出来ない場合
		cur_dist_r = 999;
	}
	if(dist_f ==0 &&(cur_dist_r-dist_r) <= 1&&(cur_dist_r-dist_r) >= -1){
			return 1;
	}
	if(dist_r ==0 &&(cur_dist_f-dist_f) <= 1&&(cur_dist_f-dist_f) >= -1){
			return 1;
	}

	if((cur_dist_f-dist_f) <= 1&&(cur_dist_f-dist_f) >= -1 && (cur_dist_r-dist_r) <= 1&&(cur_dist_r-dist_r) >= -1 ){
			return 1;
	}else{
			return -1;
	}

}

/*
		ref_dist_f = dist_f;
		ref_dist_r = dist_r;
		uss_f_l=uss_get(1); // f_left
		usleep(50000);
		uss_f_r=uss_get(2); // f_right
		usleep(50000);
		uss_r_l=uss_get(3); // right_l
		usleep(50000);
		uss_r_r=uss_get(4); // right_r
		usleep(50000);

		//両方のUSSの距離を信頼する場合
		if((uss_f_l > uss_min  || uss_f_l < uss_max ) && (uss_f_r > uss_min  || uss_f_r < uss_max)){
			cur_dist_f =(uss_f_l + uss_f_r)/2;
		} // 左のUSSの距離を信頼する場合
		else if(uss_f_l > uss_min  || uss_f_l < uss_max){
			cur_dist_f = uss_f_l;
		} // 右のUSSの距離を信頼する場合
		else if(uss_f_r > uss_min  || uss_f_r < uss_max){
			cur_dist_f = uss_f_r;
		}
		else{ //両方とも信頼出来ない場合
			cur_dist_f = 999;
		}

		//右側のUSSの距離を信頼する場合
		if((uss_r_l > uss_min  || uss_r_l < uss_max ) && (uss_r_r > uss_min  || uss_r_r < uss_max)){
			cur_dist_r =(uss_r_l + uss_r_r)/2;
		} // 左のUSSの距離を信頼する場合
		else if(uss_r_l > uss_min  || uss_r_l < uss_max){
			cur_dist_r = uss_r_l;
		} // 右のUSSの距離を信頼する場合
		else if(uss_r_r > uss_min  || uss_r_r < uss_max){
			cur_dist_r = uss_r_r;
		}
		else{ //両方とも信頼出来ない場合
			cur_dist_r = 999;
		}

		printf("ref_dist_f=%d     cur_dist_f = %d\n",ref_dist_f,cur_dist_f);
		printf("ref_dist_r=%d     cur_dist_r = %d\n",ref_dist_r,cur_dist_r);
		if(cur_dist_f!=999 && cur_dist_f!=999){
			//移動
           	run_ctrl_set_xy_dist((float)(cur_dist_f - ref_dist_f),(float)(cur_dist_r - ref_dist_r));
			run_ctrl_create();
			while(1){
				//printf("run straight loop \n");
				if(run_ctrl_get_endflag()==1){
					//run_ctrl_reset();
					break;
				}
				usleep(20000);
			}
			run_ctrl_cancel();

		}else{
			printf("can't get distance using USS\n");
			return -1;
		}
*/
/*
//再測定
		uss_f_l=uss_get(1); // f_left
		usleep(50000);
		uss_f_r=uss_get(2); // f_right
		usleep(50000);
		uss_r_l=uss_get(3); // right_l
		usleep(50000);
		uss_r_r=uss_get(4); // right_r
		usleep(50000);
		//両方のUSSの距離を信頼する場合
		if((uss_f_l > uss_min  || uss_f_l < uss_max ) && (uss_f_r > uss_min  || uss_f_r < uss_max)){
			cur_dist_f =(uss_f_l + uss_f_r)/2;
		} // 左のUSSの距離を信頼する場合
		else if(uss_f_l > uss_min  || uss_f_l < uss_max){
			cur_dist_f = uss_f_l;
		} // 右のUSSの距離を信頼する場合
		else if(uss_f_r > uss_min  || uss_f_r < uss_max){
			cur_dist_f = uss_f_r;
		}
		else{ //両方とも信頼出来ない場合
			cur_dist_f = 999;
		}

		//右側のUSSの距離を信頼する場合
		if((uss_r_l > uss_min  || uss_r_l < uss_max ) && (uss_r_r > uss_min  || uss_r_r < uss_max)){
			cur_dist_r =(uss_r_l + uss_r_r)/2;
		} // 左のUSSの距離を信頼する場合
		else if(uss_r_l > uss_min  || uss_r_l < uss_max){
			cur_dist_r = uss_r_l;
		} // 右のUSSの距離を信頼する場合
		else if(uss_r_r > uss_min  || uss_r_r < uss_max){
			cur_dist_r = uss_r_r;
		}
		else{ //両方とも信頼出来ない場合
			cur_dist_r = 999;
		}

*/
/*		usleep(1000*1000); //1秒待つ

		//地図プログラムに補正位置を送る。(壁との距離ですよ)

//		uss_f_l

	return 1;
*/

