/*

走行制御
指令値から3つのタイヤの速度値を計算し速度管理に送る
6方向のみに進む

*/
#include <stdio.h>
#include <linux/errno.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/io.h>
#include "./../hardware.h"
#include "./../run/vel_ctrl.h"
#include "./../run/distance.h"
#include "hexagon.h"
#include <math.h>//座標計算用

static pthread_t task_id;
static float run_speed;
static int dist_flag=0;
static float run_x_dist, run_y_dist, run_shet_dist;
static float run_dist=0;
static int run_mode = 0;
static int ctrl_endflag=0;

//static enum run_mode run_mode=STRAIGHT;

// Kp_s : 直進制御において、左右のタイヤの距離差に対する補正速度の係数
// Kp_r : 回転制御において、左右のタイヤの距離差に対する補正速度の係数
// Kf_s : 直進停止制御において、残り距離に対する速度指定値を決める係数
// Kf_r : 回転停止制御において、残り角度に対する速度指定値を決める係数
static float Kp_s = 1.0; //1.0 ;// 3.0;
static float Kp_r = 0.5; //1.0 ;// 3.0;
static float Kf_s = 1.0;//1.0
static float Kf_r = 0.3;
static float Kf_r_org;

static int thread_create();
static void set_mode(int mode);//移動方向を決定する関数

static void *thread_code(void *t){

	int counter=0;
	struct distance_data distance_data;

	float DIST_vel_down = 10; //直進停止制御において、減速を開始する残り距離
	float ANGLE_vel_down = 20; //回転停止制御において、減速を開始する残り角度
	float DIST_vel_stop = 8; //直進停止制御において、停止を開始する残り距離
	float ANGLE_vel_stop = 5; //回転停止制御において、停止を開始する残り角度

	float curr_x_dist,curr_y_dist,curr_shet_dist;//現在の移動距離
    float dist_x_diff,dist_y_diff,dist_shet_diff;//目標までの距離の差
	float vel_x_speed=0,vel_y_speed=0,vel_shet_speed=0;//速度、移動方向の速度
	float vel_r_speed,vel_l_speed,vel_b_speed;//実際にタイヤに与える速度
	//角速度変換用
	float coef_r;
	coef_r = 1./DISTANCE_TIRE * 180./ 3.1415;
	float rot_coef;
	//rot_coef = DISTANCE_TIRE * 180./ 3.1415;
	rot_coef = DISTANCE_TIRE / 180.* 3.1415;

	//タイヤ走行距離を車体回転角度に変換する際の係数
	//coef = 1. /BETWEEN_TIRE  * 180 / 3.1415;
	//float srip_down = 0.2;
	Kf_r_org = Kf_r;
	
	run_shet_dist = 0;

	while(1){
		counter++;

		distance_get(&distance_data);

			if(dist_flag==1){
				set_mode(run_mode);//移動方向を決定する。（whileの外かもしれない）
				//距離データの計算
				curr_x_dist = distance_data.x_dist;
				curr_y_dist = distance_data.y_dist;
				dist_x_diff = run_x_dist - curr_x_dist;
				dist_y_diff = run_y_dist - curr_y_dist;
				curr_shet_dist = distance_data.shet;
				dist_shet_diff = run_shet_dist - curr_shet_dist;				

				//x方向の速度を決める
				if(run_x_dist > 0){//正方向への移動
					vel_x_speed= run_speed;
					if( dist_x_diff < DIST_vel_down){//指定位置に近づいてきた
						//vel_x_speed = Kf_s * dist_x_diff  + 2;
						//vel_x_speed = Kf_s * dist_x_diff;
						vel_x_speed = Kf_s * dist_x_diff/fabsf(dist_x_diff) * run_speed;
					}	
					
					if(dist_x_diff <= 0 && fabsf(dist_x_diff) < DIST_vel_stop){//目標値到達
					//if(fabsf(dist_x_diff) < DIST_vel_stop){//目標値到達
						vel_x_speed = 0;//stop
					}
				}else if(run_x_dist < 0){//負方向への移動
					vel_x_speed = -run_speed;
					if( dist_x_diff > -DIST_vel_down){
						//vel_x_speed = Kf_s * dist_x_diff - 2;
						// diff<0
						//vel_x_speed = Kf_s * dist_x_diff;
						vel_x_speed = Kf_s * dist_x_diff/fabsf(dist_x_diff) * run_speed;
					}
					if(dist_x_diff >= 0 && fabsf(dist_x_diff) < DIST_vel_stop){//目標値到達
					//if(fabsf(dist_x_diff)< DIST_vel_stop){//目標値到達
						vel_x_speed = 0;
					}
				}else {//x方向の移動なし
					vel_x_speed = 0;
					if(fabsf(dist_x_diff) > DIST_vel_stop){//位置のずれに対して補正をかける
						vel_x_speed = Kf_s * dist_x_diff/fabsf(dist_x_diff) * run_speed;
					}
				
				}
				
				//y方向の速度を決める
				if(run_y_dist > 0){//正方向への移動
					vel_y_speed = run_speed;
					if( dist_y_diff < DIST_vel_down){//指定位置に近づいてきた
						//vel_y_speed = Kf_s * dist_y_diff + 2;
						//vel_y_speed = Kf_s * dist_y_diff;
						vel_y_speed = Kf_s * dist_y_diff/fabsf(dist_y_diff) * run_speed;
					}
					if(dist_y_diff <= 0 && fabsf(dist_y_diff)< DIST_vel_stop){//目標値到達
					//if(fabsf(dist_y_diff)< DIST_vel_stop){//目標値到達
						vel_y_speed = 0;
					}
				}else if(run_y_dist < 0){//負方向への移動
					vel_y_speed = -run_speed;
					if( dist_y_diff > -DIST_vel_down){
						//vel_y_speed = Kf_s * dist_y_diff - 2;
						//vel_y_speed =  Kf_s * dist_y_diff;
						vel_y_speed = Kf_s * dist_y_diff/fabsf(dist_y_diff) * run_speed;
					}
					if(dist_y_diff >= 0 && fabsf(dist_y_diff)< DIST_vel_stop){//目標値到達
					//if(fabsf(dist_y_diff)< DIST_vel_stop){//目標値到達
						vel_y_speed = 0;
					}
				}else {//y方向の移動なし
					vel_y_speed = 0;
					if(fabsf(dist_y_diff) > DIST_vel_stop){//位置がずれた
						vel_y_speed = Kf_s * dist_y_diff/fabsf(dist_y_diff) * run_speed;
					}

				}

				//shet方向の速度を決める
				if(run_shet_dist > 0){//正方向への移動
					vel_shet_speed= run_speed * rot_coef;
					if( dist_shet_diff < ANGLE_vel_down){//指定位置に近づいてきた
						//vel_shet_speed = Kp_s * dist_shet_diff + 2;
						//vel_shet_speed = Kp_s * dist_shet_diff * rot_coef;
						vel_shet_speed = Kp_s * dist_shet_diff/fabsf(dist_shet_diff) * run_speed * rot_coef;
					}
					if(dist_shet_diff <= 0 && fabsf(dist_shet_diff)< ANGLE_vel_stop){//目標値到達				
					//if(fabsf(dist_shet_diff)< ANGLE_vel_stop){//目標値到達
						vel_shet_speed = 0;
					}
				}else if(run_shet_dist < 0){//負方向への移動
					vel_shet_speed = -run_speed * rot_coef;
					if( dist_shet_diff > -ANGLE_vel_down){
						//vel_shet_speed = -Kp_s * dist_shet_diff - 2;
						//vel_shet_speed = Kp_s * dist_shet_diff * rot_coef;
						vel_shet_speed = Kp_s * dist_shet_diff/fabsf(dist_shet_diff) * run_speed * rot_coef;
					}
					if(dist_shet_diff >= 0 && fabsf(dist_shet_diff)< ANGLE_vel_stop){//目標値到達				
					//if(fabsf(dist_shet_diff)< ANGLE_vel_stop){//目標値到達
						vel_shet_speed = 0;
					}
				}else {//shet方向の移動なし
					vel_shet_speed = 0;
					if(fabsf(dist_shet_diff) > ANGLE_vel_stop){//ずれが発生
						vel_shet_speed = Kp_s * dist_shet_diff/fabsf(dist_shet_diff) * run_speed * rot_coef;
					}
				}
			}


		//if(vel_x_speed = 0 && vel_y_speed == 0 && vel_shet_speed == 0){
		if(vel_x_speed < 1 && vel_x_speed > -1 && vel_y_speed < 1 && vel_y_speed > -1 && vel_shet_speed < 1 && vel_shet_speed > -1){
			vel_ctrl_set(0., 0., 0.);
			ctrl_endflag=1;
			return 0;
			goto END;//終了

		}
		printf("x=%f,y=%f,shet=%f,\n",vel_x_speed,vel_y_speed,vel_shet_speed);
		//座標変換
		vel_l_speed = - vel_x_speed/2 - vel_y_speed * sqrt(3.0)/2.0 + vel_shet_speed;
		vel_r_speed = - vel_x_speed/2 + vel_y_speed * sqrt(3.0)/2.0 + vel_shet_speed;
		vel_b_speed =   vel_x_speed   + vel_shet_speed;
		vel_ctrl_set(vel_l_speed, vel_r_speed, vel_b_speed);//速度を与える

		usleep((T_RUN_CTRL - 1)*1000); // 走行制御（直進、回転、旋回）

	}

	END:
	vel_ctrl_set(0., 0., 0.);
	ctrl_endflag=1;

	return 0;

}


static int thread_create()
{
	pthread_attr_t attr;
	struct sched_param sched_param;
	int ret;
	int policy;

	pthread_attr_init(&attr);

	sched_param.sched_priority = 20;
	pthread_attr_setschedparam(&attr, &sched_param);
	pthread_setschedparam(pthread_self(), SCHED_FIFO , &sched_param);

	ret = pthread_create(&task_id, &attr, thread_code, (void *)2);

	pthread_getschedparam(pthread_self(), &policy, &sched_param);
	printf("plicy = %s, priority =%d\n",
		(policy == SCHED_FIFO)  ? "SCHED_FIFO" :
		(policy == SCHED_RR)    ? "SCHED_RR" :
		(policy == SCHED_OTHER) ? "SCHED_OTHER" :
		"???",
		sched_param.sched_priority);

	return 0;
}


void hexagon_create(){

	ctrl_endflag=0;
	distance_reset();
	thread_create();

}

int hexagon_get_endflag(){

	return ctrl_endflag;
}

void hexagon_cancel(){

	ctrl_endflag=0;
	pthread_cancel(task_id);

}

void hexagon_reset(){

	distance_reset();
	//ctrl_endflag=0;
	//dist_flag = 0;

}

void hexagon_set_dist_mode(float dist,int mode){

	ctrl_endflag=0;
	dist_flag = 1;
	run_dist = dist;
	run_mode = mode;

}

void hexagon_unset_dist(){
	run_x_dist = 0;
	run_y_dist = 0;
	run_shet_dist = 0;
	dist_flag = 0;
}

void hexagon_stop(){
	vel_ctrl_set(0., 0., 0.);
}

void hexagon_set_Kfr(float Kfr){
	Kf_r  =  Kfr;
}

void hexagon_reset_Kfr(){
	Kf_r  =  Kf_r_org;
}

void hexagon_set_speed(float speed){

	run_speed = speed;
}

void set_mode(int mode){
	switch(mode){
		case 0 ://ストップモード
			run_x_dist = 0;
			run_y_dist = 0;
			break;
		case 1 ://直進 後退
			run_x_dist = 0;
			run_y_dist = run_dist;
			break;
		case 2 ://右斜め移動
			run_x_dist = run_dist * sqrt(3.0)/2.0;
			run_y_dist = run_dist /2.0;
			break;
		case 3 ://左斜め移動
			run_x_dist = -run_dist * sqrt(3.0)/2.0;
			run_y_dist = run_dist /2.0;
			break;

		default :
			run_x_dist = 0;
			run_y_dist = 0;
			break;

	}

}

