/*
距離計


3輪の距離データからx,y シータの三つの距離データに変換する

by Y.Igarashi
修正　K.Mano
*/
#include <stdio.h>
#include <linux/errno.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/io.h>
#include "./../hardware.h"
#include "./../device/encoder.h"
#include "distance.h"
#include <math.h>//計算用

 
extern int run_ctrl_mode_dir;
extern const int run_ctrl_strate_mode;
extern const int run_ctrl_right_mode;
extern const int run_ctrl_left_mode;
extern int run_ctrl_right_mode_back;
extern int run_ctrl_left_mode_back;


static float dis_coef = 2*3.1415*TIRE_R/ENC_PPS/M_E_RATIO; //for STANDARD MIRS
//static float dis_coef_b = 2*3.1415*TIRE_R/ENC_PPS/M_E_RATIO_B; //for MIRS bad pu-ri
static float SLIP_Y = 0.5;//タイヤのスリップを考慮するための値
static float SLIP_X = 0.6;
static float SLIP_R = 0.40;//タイヤのスリップを考慮するための値 回転量
static float coef_r = 1./DISTANCE_TIRE * 180./ 3.1415;//回転量を角度に変換する

static const float slip_y_strate	= 0.52;	//54strate mode's slip
static const float slip_x_strate 	= 0.54;	//56

static const float slip_y_right 	= 0.54;	//57right30 mode's slip
static const float slip_x_right 	= 0.61;	//59

static const float slip_y_left 		= 0.56;	//54 left mode's slip
static const float slip_x_left 		= 0.52;//46	

static const float slip_y_right_back 	= 0.60;	//right30_back mode's slip
static const float slip_x_right_back 	= 0.45;	

static const float slip_y_left_back	= 0.59;	//58 left30_back mode's slip
static const float slip_x_left_back	= 0.52;//48	


void distance_get(struct distance_data *distance_data){

	struct encoder_data encoder_data;

//select slip mode
	if(run_ctrl_mode_dir == run_ctrl_strate_mode) {
		SLIP_X = slip_x_strate;
		SLIP_Y = slip_y_strate;
	}else if(run_ctrl_mode_dir == run_ctrl_right_mode){
		SLIP_X = slip_x_right;
		SLIP_Y = slip_y_right;
	}else if(run_ctrl_mode_dir == run_ctrl_left_mode){
		SLIP_X = slip_x_left;
		SLIP_Y = slip_y_left;
	}else if(run_ctrl_mode_dir == run_ctrl_right_mode_back){
		SLIP_X = slip_x_right_back;
		SLIP_Y = slip_y_right_back;
	}else if(run_ctrl_mode_dir == run_ctrl_left_mode_back){
		SLIP_X = slip_x_left_back;
		SLIP_Y = slip_y_left_back;
	}else{
	 	SLIP_X = slip_x_strate;	//default
		SLIP_Y = slip_y_strate;
	}

	//printf("mode dir = %d\n",run_ctrl_mode_dir);
/*
	switch(run_ctrl_mode_dir){
	case run_ctrl_strate_mode:
		SLIP_X = slip_x_strate;
		SLIP_Y = slip_y_strate;
		break;
	case run_ctrl_right_mode:
		SLIP_X = slip_x_right;
		SLIP_Y = slip_y_right;
		break;
	case run_ctrl_left_mode:
		SLIP_X = slip_x_left;
		SLIP_Y = slip_y_left;
		break;
	default:
 		SLIP_X = slip_x_strate;
		SLIP_Y = slip_y_strate;
	}
*/
	encoder_get(&encoder_data);

	//エンコーダ積算値から距離[cm]に変換
	//TIRE_DIST_RATIO で左右の違いを吸収
/*
	distance_data->left = encoder_data.left_sum * dis_coef * TIRE_DIST_RATIO ;
	distance_data->right = encoder_data.right_sum * dis_coef;
*/
	float right_dist,left_dist,back_dist;
//	right_dist = encoder_data.left_sum * dis_coef * TIRE_DIST_RATIO ;//右タイヤの回転量
	right_dist = encoder_data.left_sum * dis_coef;//右タイヤの回転量
	left_dist = encoder_data.right_sum * dis_coef  * ENC_L;	//左タイヤの回転量
	back_dist = encoder_data.back_sum * dis_coef * ENC_B;	//後ろタイヤの回転量
	//座標変換 SLIPはスリップを考慮して移動量を減少させる値
	distance_data->x_dist = (-right_dist/2.0 -left_dist/2.0 + back_dist)*SLIP_X;//x軸方向の移動量
	distance_data->y_dist = (-right_dist*sqrt(3.0)/2.0 + left_dist*sqrt(3.0)/2.0)*SLIP_Y;//y軸方向の移動量
//	distance_data->shet = right_dist + left_dist + back_dist;//回転量[cm]
	distance_data->shet = (right_dist + left_dist + back_dist) * coef_r* SLIP_R;//回転量[deg]
	//printf("distance_l =%6.1f     distance_r=%6.1f\n",distance_data->left, distance_data->right);
	return;

}

void distance_sum_get(struct distance_data *distance_data_all){

	struct encoder_all_data encoder_all_data;

	encoder_all_get(&encoder_all_data);

	float right_dist,left_dist,back_dist;

	right_dist = encoder_all_data.left_sum_cum * dis_coef;//右タイヤの回転量
	//right_dist = encoder_all_data.left_sum_cum * dis_coef * TIRE_DIST_RATIO;//右タイヤの回転量
	//left_dist = encoder_all_data.right_sum_cum * dis_coef_b;	//左タイヤの回転量
	left_dist = encoder_all_data.right_sum_cum * dis_coef ;	//左タイヤの回転量
	back_dist = encoder_all_data.back_sum_cum * dis_coef ;	//後ろタイヤの回転量
	//座標変換
	distance_data_all->x_dist = (-right_dist/2.0 -left_dist/2.0 + back_dist)* SLIP_X; //x軸方向の移動量
	distance_data_all->y_dist = (-right_dist*sqrt(3.0)/2.0 + left_dist*sqrt(3.0)/2.0) * SLIP_Y;//y軸方向の移動量
//	distance_data_all->shet = right_dist + left_dist + back_dist;//回転量
	distance_data_all->shet = (right_dist + left_dist + back_dist) * coef_r * SLIP_R;//回転量[deg]
	return;

}

void distance_reset(){

	encoder_reset();
	return;

}
void distance_all_reset(){
	encoder_all_reset();
	return;

}
